/*
  MLink-FrSky translator from T.Pfaff 29.10.2016 V0.2 based on FrSky S-Port Telemetry library
  (c) Pawelsky 20160916
  Not for commercial use
  
  Only für ATmega 328P (Arduino Uno o. ProMini 5V)
*/
#include <MLinkExMin.h>
#include <avr/io.h>
#include <avr/wdt.h>
MLinkExMin mLink(13); //LED 13 blinks on every MLink data set

//Only if MPX Tx-Modul is used uncomment this, comment it for ACT-Tx
//#define _MPX_TX

#include "FrSkySportSensor.h"
#include "FrSkySportSensorAss.h"
#include "FrSkySportSensorFcs.h"
#include "FrSkySportSensorFlvss.h"
//#include "FrSkySportSensorGps.h"
#include "FrSkySportSensorRpm.h"
#include "FrSkySportSensorSp2uart.h"
#include "FrSkySportSensorVario.h"
#include "FrSkySportSingleWireSerial.h"
#include "FrSkySportTelemetry.h"
#include "SoftwareSerial.h"


FrSkySportSensorAss ass;                               // Create ASS sensor with default ID
FrSkySportSensorFcs fcs1;                               // Create FCS-40A sensor with default ID (use ID8 for FCS-150A)
FrSkySportSensorFcs fcs2;                               // Create FCS-40A sensor with default ID (use ID8 for FCS-150A)
FrSkySportSensorFlvss flvss1;                          // Create FLVSS sensor with default ID
FrSkySportSensorFlvss flvss2(FrSkySportSensor::ID15);  // Create FLVSS sensor with given ID
//FrSkySportSensorGps gps;                               // Create GPS sensor with default ID
FrSkySportSensorRpm rpm;                               // Create RPM sensor with default ID
FrSkySportSensorSp2uart sp2uart;                       // Create SP2UART Type B sensor with default ID
FrSkySportSensorVario vario;                           // Create Variometer sensor with default ID
FrSkySportTelemetry telemetry(true);                 // Create telemetry object with polling

void setup()
{
//Initialisation für MPX-HF by D. Kammerer 26.01.2016 (not yet tested - waiting für feetback) - not necessary for ACT-HF
#ifdef _MPX_TX

  // Nach dem Start 5Sekunden warten und nichts machen, damit das 
  // Sendemodul seinen normalen Betrieb aufnehmen kann.
  delay(5000);  
    
  // ********************************************************************
  // ** Hier auf den naechste Frame warten und dann aufsynchronisieren **
  // ********************************************************************
  pinMode(0, INPUT);
  // Warten, bis die RX-Leitung M-Link auf Low geht. Denn dann koennte ein Frame anfangen.
  do 
    {
      temp = digitalRead(0);
    } while (temp == 1);
  // Nun 3ms warten, dann sind die 23Bytes des COM-Ports 
  // 115kBaud auf jedenfall sicher Uebertragen. 
  // 23Bytes dauern ca. 2    ms
  delay(4); // Nach dieser Zeit muessten nun alle Bytes uebertragen sein.
#endif

  
   telemetry.begin(FrSkySportSingleWireSerial::SOFT_SERIAL_PIN_2, &ass, &fcs1, &fcs2, &flvss1, &flvss2,/* &gps,*/ &rpm, &sp2uart, &vario);

   wdt_enable(WDTO_1S);
   Serial.begin(38400);
   wdt_reset();

   
}
//--------------------------------------------------------------------------------------------
void loop()
{

   static float v_hor=-1;
   static float v_vert=-500,v_vert_senden;
   static float strom1=0;
   static float strom2=0;
   static float u_empf=-1;
   static float u_empf_alt=100000;
   static float u_min=100000;   
   static float u_max=0;   
   static float u_akku[16]={-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
   static float u_akku_aus[16]={-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
   static float temp1=-273;
   static float temp2=-273;
   static float hoch=-5000,hoch_senden;
   static float entfernung=-1;
   static float kap=-1;
   static float druck=-1;
   static uint32_t rssi=1,rssi_aus=100,rssi_mittel[11]={100,100,100,100,100,100,100,100,100,100,100}; // wenn mit 0 initialisiert und fehlt in MLink - wird kein FrSky update gefahren
   static float vol_rel=-1;
   static float kurs=-1;
   static uint32_t upm=-1;

   static char strom_adresse1 = -1;
   static char strom_adresse2 = -1;
   static char temp1_adresse = -1;
   static char temp2_adresse = -1;
   static char v_hor_adresse = -1;
   static char v_vert_adresse = -1;
   static char hoch_adresse = -1;
   static char entfernung_adresse = -1;
   static char kap_adresse = -1;
   static char druck_adresse = -1;
   static char rssi_adresse = -1;
   static char viol_rel_adresse = -1;
   static char kurs_adresse = -1;
   static char upm_adresse = -1;

   

    byte einheit=0;
    byte adresse=0;
    byte rssi_index=0;
    float wert=0;

    wdt_reset();
    if(mLink.get(Serial))//reading Mlink stream
    {
        if(mLink.Return.einheit_nr > 0)
        {
         wert= mLink.Return.ausgabewert;
         einheit= mLink.Return.einheit_nr;  
         adresse= mLink.Return.adresse;

        }  
          

    bool ui1_b=false,ui2_b=false,u_rssi_b=false, steighoch_b=false, upmt1t2_b=false, u_b=false;
    byte iaus=0;
    switch(einheit)
    {
      case 0x01: //Spannung [V]

        u_akku[adresse]=wert;
 
        if(adresse==0)
        {
          sp2uart.setData(u_akku[0], ((float)rssi)/100); 
         }    
        else u_b=true; 

        iaus=0;
        for(byte i = 1 ; i < 16 ; i++)
        {
           if(u_akku[i] > -1 && iaus < 12)
            u_akku_aus[iaus++] = u_akku[i];  
        } 
        u_min=100000;
        u_max=-100000;
        for(byte i = 0 ; i < iaus ; i++)
        {
          if(u_akku_aus[i] > 0 && u_akku_aus[i] <= u_min)
          {
           u_min = u_akku_aus[i];
           ui1_b=true;
          }
          if(u_akku_aus[i] > 0 && u_akku_aus[i] >= u_max )
          {
            u_max = u_akku_aus[i];
            ui2_b=true;
          }
        }
       
      break;
      case 0x02: //Strom [A]
      if(strom_adresse1 == -1 || strom_adresse1 == adresse)
      {
        ui1_b=true;
        strom_adresse1 = adresse;
        strom1 = wert;
      }  
      else if(strom_adresse2 == -1 || strom_adresse2 == adresse)
      {
        ui2_b=true;
        strom_adresse2 = adresse;
        strom2 = wert;
      }  
      break;
      case 0x03: //Steigen [m/s]
      if(v_vert_adresse == -1 || v_vert_adresse == adresse)
      {
        steighoch_b=true;
        v_vert_adresse = adresse;
        v_vert = wert;
      }  
      break;
      case 0x04: //hor. Geschw. [km/h]
      if(v_hor_adresse == -1 || v_hor_adresse == adresse)
      {
        v_hor_adresse = adresse;
        ass.setData(0.5476*wert);//correction because OTX value interpreted in unit mph
      }
      break;
      case 0x05: //upm [1/min]
      if(upm_adresse == -1 || upm_adresse == adresse)
      {
        upmt1t2_b=true;
        upm_adresse = adresse;
        upm = (uint32_t)wert;
      }  
      break;
      case 0x06: //T [°C]
      if(temp1_adresse == -1 || temp1_adresse == adresse)
      {
        upmt1t2_b=true;
        temp1_adresse = adresse;
        temp1=wert;
      }
      else if(temp2_adresse == -1 || temp2_adresse == adresse)
      {
        upmt1t2_b=true;
        temp2_adresse = adresse;
        temp2=wert;
      }
      break;
      case 0x07: //Kurs [°]
      if(kurs_adresse == -1 || kurs_adresse == adresse)
      {
        kurs_adresse = adresse;
        kurs = wert;
      }  
      break;
      case 0x08: //Höhe [m] 
      if(hoch_adresse == -1 || hoch_adresse == adresse)
      {
        steighoch_b=true;    
        hoch_adresse = adresse;
        hoch = wert;
      }  
      break;
      case 0x09: //Kapazität [%]
      break;
      case 0x0A:  //RSSI [%]
      if(rssi_adresse == -1 || rssi_adresse == adresse)
      {
        rssi_adresse = adresse;
        rssi = (uint32_t)wert;
       // rssi_aus = 100-(25/sqrt(rssi));

        if(rssi > 100)
             rssi_mittel[rssi_index++] = 100;
        else if ( rssi <=0)  
             rssi_mittel[rssi_index++] = 0;
        else  
             rssi_mittel[rssi_index++] = 100-(25/sqrt(rssi));
            
        if(rssi_index > 10) rssi_index = 0;
            
            
        for(byte index=0 ; index < 10 ; index++)
        {
              rssi_aus += rssi_mittel[index];
        }  
        rssi_aus /= 11; //*/
      }  
      break;
      case 0x0B: //Kapazität [mAh]
      break;
      case 0x0C: //Volumen [ml]
      break;
      case 0x0D: //Entfern. [km]
      break;
      case 0x0E: //Druck [hPa]
      break;
      case 0x0F: //frei [TBDF]
      break;  
    }
 



    if(ui1_b) fcs1.setData(strom1, u_min);  //crent 1 ans lowest of all vaoltages
    if(ui2_b) fcs2.setData(strom2, u_max);  //crent 2 ans heighes of all vaoltages
                   
    switch(iaus)
    {
    case 1:
      flvss1.setData(u_akku_aus[0]);    
    break;
    case 2:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1]);    
    break;
    case 3:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2]);    
    break;
    case 4:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2],u_akku_aus[3]);    
    break;
    case 5:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2],u_akku_aus[3],u_akku_aus[4]);
    break;
    case 6:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2],u_akku_aus[3],u_akku_aus[4],u_akku_aus[5]);
    break;    
    case 7:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2],u_akku_aus[3],u_akku_aus[4],u_akku_aus[5]);
      flvss2.setData(u_akku_aus[6]);
    break;
    case 8:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2],u_akku_aus[3],u_akku_aus[4],u_akku_aus[5]);
      flvss2.setData(u_akku_aus[6],u_akku_aus[7]);
    break;
    case 9:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2],u_akku_aus[3],u_akku_aus[4],u_akku_aus[5]);
      flvss2.setData(u_akku_aus[6],u_akku_aus[7],u_akku_aus[8]);
    break;
    case 10:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2],u_akku_aus[3],u_akku_aus[4],u_akku_aus[5]);
      flvss2.setData(u_akku_aus[6],u_akku_aus[7],u_akku_aus[8],u_akku_aus[9]);
    break;
    case 11:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2],u_akku_aus[3],u_akku_aus[4],u_akku_aus[5]);
      flvss2.setData(u_akku_aus[6],u_akku_aus[7],u_akku_aus[8],u_akku_aus[9],u_akku_aus[10]);
    break;
    case 12:
      flvss1.setData(u_akku_aus[0],u_akku_aus[1],u_akku_aus[2],u_akku_aus[3],u_akku_aus[4],u_akku_aus[5]);
      flvss2.setData(u_akku_aus[6],u_akku_aus[7],u_akku_aus[8],u_akku_aus[9],u_akku_aus[10],u_akku_aus[11]);
    break;
  }


  // Set GPS data
//  gps.setData(48.858289, 2.294502,   // Latitude and longitude in degrees decimal (positive for N/E, negative for S/W)
//              245.5,                 // Altitude in m (can be negative)
//              100.0,                 // Speed in m/s
//              90.23,                 // Course over ground in degrees (0-359, 0 = north)
//              14, 9, 14,             // Date (year - 2000, month, day)
//              12, 00, 00);           // Time (hour, minute, second) - will be affected by timezone setings in your radio
//
  // Set RPM/temperature sensor data
  // (set number of blades to 2 in telemetry menu to get correct rpm value)
    if(upmt1t2_b)
    {
      rpm.setData(0.5*upm, temp1, temp2);
    }
    if(steighoch_b)
    {
      if(v_vert < 0) //Offset in OTX for v_vert and hoch
        v_vert_senden = -(1638.4+v_vert);
      else
        v_vert_senden = v_vert;
     
      if(hoch < 0)
        hoch_senden = -(16133.5+hoch);
      else
        hoch_senden =  hoch + 250.5;
      vario.setData(hoch_senden, v_vert_senden); 
    }  
    telemetry.send(rssi_aus);
}  
}
